//  <(((*> <(((*> ><(((*> ><(((*> ><(((*> ><(((*> ><(((*> ><(((*> ><(((*>
//
//          appImageHelper
//          Copyright (C) 2020 Ingemar Ceicer
//          http://ceicer.org/ingemar/
//          programmering1 (at) ceicer (dot) org
//
//   This program is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License version 3
//   as published by the Free Software Foundation.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
// <(((*> <(((*> ><(((*> ><(((*> ><(((*> ><(((*> ><(((*> ><(((*> ><(((*>

#include "checkforupdates.h"
#include "info.h"
#include "widget.h"

void CheckForUpdates::check(const QString &progName,
                            const QString &currentVersion,
                            const QString &versionPath,
                            const QString &downloadPath)
{
    bool *doUpdate = new bool(false);
    QUrl url(versionPath);
    auto *nam = new QNetworkAccessManager(nullptr);
    auto *reply = nam->get(QNetworkRequest(url));
    QObject::connect(
        reply, &QNetworkReply::finished,
    [this, reply, versionPath, progName, currentVersion, downloadPath, doUpdate]() {
        QByteArray bytes = reply->readAll(); // bytes
        QString newVersion(bytes);           // string
        newVersion = newVersion.trimmed();
        int index = newVersion.indexOf('\n');
        QString onlyNewVersion = newVersion.left(index);
        QString infoVersion = newVersion.mid(index);
        QString info = "";

        if(index > 0) {
            info = moreInfo(&infoVersion);
        }

        if(onlyNewVersion == "") {
            QMessageBox::critical(
                nullptr, progName + " " + currentVersion,
                tr("No Internet connection was found.\nPlease check your "
                   "Internet settings and firewall."));
            return;
        } else {
            int ver = jfrVersion(currentVersion, onlyNewVersion);

            switch(ver) {
                case 0:
                    *doUpdate = true;
                    break;

                case 1:
                    QMessageBox::information(
                        nullptr, progName + " " + currentVersion,
                        tr("Your version of ") + progName +
                        tr(" is newer than the latest official version. ") + "(" +
                        onlyNewVersion + ").");
                    *doUpdate = false;
                    break;

                case 2:
                    QMessageBox::information(
                        nullptr, progName + " " + currentVersion,
                        tr("Your version of ") + progName +
                        tr(" is the same version as the latest official version. ") + "(" +
                        onlyNewVersion + ").");
                    *doUpdate = false;
                    break;

                default:
                    QMessageBox::information(
                        nullptr, progName + " " + currentVersion,
                        tr("There was an error when the version was checked."));
                    *doUpdate = false;
                    break;
            }
        }

        emit sendValue(*doUpdate);
        close();
    });
}

void CheckForUpdates::checkOnStart(const QString &progName,
                                   const QString &currentVersion,
                                   const QString &versionPath,
                                   const QString &downloadPath)
{
    QUrl url(versionPath);
    auto *nam = new QNetworkAccessManager(nullptr);
    auto *reply = nam->get(QNetworkRequest(url));
    QObject::connect(
        reply, &QNetworkReply::finished,
    [this, reply, versionPath, progName, currentVersion, downloadPath]() {
        QByteArray bytes = reply->readAll(); // bytes
        QString newVersion(bytes);           // string
        newVersion = newVersion.trimmed();
        int index = newVersion.indexOf('\n');
        QString onlyNewVersion = newVersion.left(index);
        QString infoVersion = newVersion.mid(index + 1);
        QString info = "";

        if(index > 0) {
            info = moreInfo(&infoVersion);
        }

        if(onlyNewVersion == "") {
            QMessageBox::critical(
                nullptr, progName + " " + currentVersion,
                tr("No Internet connection was found.\nPlease check your "
                   "Internet settings and firewall."));
        } else {
            int ver = jfrVersion(currentVersion, onlyNewVersion);

            if(ver == 0) {
                QString updateinstruktions = "<i>" + tr("Update ") + DISPLAY_NAME + ":</i> <br>" + tr("Click \"Update\"<br><br>For versions earlier than 2.4.0, download a new AppImage from<br><a href=\"https://gitlab.com/posktomten/appimagehelper/-/wikis/DOWNLOADS\">https://gitlab.com/posktomten/appimagehelper/-/wikis/DOWNLOADS</a><br>Versions from 2.4.0 are self-updating.");
                QMessageBox::information(
                    nullptr, progName + " " + currentVersion,
                    "<h2 style=\"color:green\">" +
                    tr("There is a new version of ") + progName +
                    "</h2><h3 style=\"color:green\">" + tr("Latest version: ") +
                    onlyNewVersion + "</h3<p>" + updateinstruktions + info +
                    "</p>");
            } else if(ver == 3) {
                QMessageBox::information(
                    nullptr, progName + " " + currentVersion,
                    tr("\nThere was an error when the version was checked."));
            }
        }
    });
    close();
}

int CheckForUpdates::jfrVersion(const QString &currentVersion,
                                const QString &newVersion)
{
    int newVer;
    int currentVer;
    bool ok = true;
    QStringList listCurrentVersion = currentVersion.split(".");
    QStringList listNewVersion = newVersion.split(".");

    for(int i = 0; i <= 2; i++) {
        currentVer = listCurrentVersion.at(i).toInt(&ok, 10);
        newVer = listNewVersion.at(i).toInt(&ok, 10);

        if(!ok)
            return 3;

        if(newVer > currentVer)
            return 0;

        if(newVer < currentVer)
            return 1;
    }

    return 2;
}
QString CheckForUpdates::moreInfo(const QString *infoVersion)
{
    QStringList listInfo;
    listInfo = infoVersion->split('\n');
    QString news;

    if(!listInfo.empty()) {
        news = "<br><br><i>" + tr("Updates:") + "</i>";
        news.append("<ol>");

        for(int i = 0; i < listInfo.size(); i++) {
            news.append("<li>" + listInfo.at(i) + "</li>");
        }
    } else {
        news = "";
    }

    news.append("</ol>");
    return news;
}
CheckForUpdates::CheckForUpdates()
{
    this->setAttribute(Qt::WA_DeleteOnClose, true);
}
CheckForUpdates::~CheckForUpdates()
{
    // delete this;
}
